## Copyright (C) 2007 John W. Eaton
##
## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {[@var{x}, @var{obj}, @var{info}]} fminunc (@var{f}, @var{x0}, @var{options})
## Minimize the function @var{f} starting from @var{x0}.
## @end deftypefn

## These are the meanings of info for Matlab's fminunc:
##
##  1: Magnitude of gradient smaller than the specified tolerance.
##  2: Change in x was smaller than the specified tolerance.
##  3: Change in the objective function value was less than the specified
##     tolerance. 
##  0: Number of iterations exceeded options.MaxIter or number of
##     functionevaluations exceeded options.FunEvals.
## -1: Algorithm was terminated by the output function.
## -2: Line search cannot find an acceptable point along the current
##     search direction.
##
## These are the meanings of info for Octave's sqp:
##
##   0: The problem is feasible and convex.  Global solution found.
##   1: The problem is not convex.  Local solution found.
##   2: The problem is not convex and unbounded.
##   3: Maximum number of iterations reached.
##   6: The problem is infeasible.
## 101: norm of last ste less than tol * norm (x)
## 102: BFGS update failed
## 103: iteration limit reached
##
## These are the meanings of info for NPSOL:
##
##  0: optimal solution found
##  1: weak local solution found
##  2: no feasible point for linear constraints and bounds
##  3: no feasible point found for nonlinear constraints
##  4: iteration limit reached
##  6: current point cannot be improved upon
##  7: user-supplied derivatives appear to be incorrect
##  9: internal error: invalid input parameter
##
## The matches are not perfect, so we do the best we can.

function [x, obj, info] = fminunc (f, x0, options)

  if (nargin == 2 || nargin == 3)

    ## We don't bother to check options for GradObj.  Just use the
    ## second element of f if it is provided.  Note also that we are not
    ## attempting to handle the case of a single function that returns
    ## both the objective and gradient.

    if (isempty (getenv ("OCTAVE_FMINUNC_USE_NPSOL")))
      [x, obj, sqp_info] = sqp (x0, f);
      switch (sqp_info)
	case 0, info = 1;
	case 1, info = 2;
	case 2, -7;
	case 3, -8;
	case 6, -9;
	case 101, info = 2;
	case 102, info = -10;
	case 103, info = 0;
      endswitch
    else
      [x, obj, npsol_info] = npsol (x0, f);
      switch (npsol_info)
	case 0, info = 1;
	case 1, info = 2;
	case 2, info = -3;
	case 3, info = -4;
	case 4, info = 0;
	case 6, info = 2;
	case 7, info = -5;
	case 9, info = -6;
      endswitch
    endif
  else
    usage ("[x, obj, info] = retval = fminunc (f, x0)");
  endif

endfunction

%!function obj = phi (x)
%!  obj = (1-x(1))^2 + 100*(x(2)-x(1)^2)^2;
%!
%!function grd = grad (x)
%!  tmp = 200*(x(2)-x(1)^2);
%!  grd = [2*(x(1)-1)-2*x(1)*tmp; tmp];
%!
%!test
%! options = optimset ("GradObj", "on");
%! [x, fval] = fminunc ({@phi, @grad}, [1; 1], options);
%! assert (x, [1; 1]);
%! assert (fval, 0);
